__version__ = '0.5.0'

PORTAGE_TMPDIR = '/var/tmp/portage'  # nosec
TMPLOGFILE = '/var/log/portage/tmerge/tmp.emerge.log'
TMERGE_LOGFILE = '/var/log/portage/tmerge/emerge.log'
EMERGE_LOGFILE = '/var/log/emerge.log'
DEVNULL = '/dev/null 2>&1'
